/*
 *  NAME
 *	libnew.h ($Revision: 3.1 $)
 * 
 *  DESCRIPTION
 *	Include file to handle the access to the LWP(3) library and to define
 *	the things needed for libonew.c/lwp_new.c
 *
 *	The following definements are used/set:
 *	    NO_LWP	no LWP library wanted
 *	    USE_LIBLWP	use the LWP library
 *	    NO_DBUG	don't use the DBUG library
 *
 *  NOTE
 *	Don't forget to link with '-lwp' if you use the LWP library
 *	(Sun only).
 *
 *	The DBUG library is only available for Unix systems. So for other
 *	systems the used macros are faded out. If you don't have it compile
 *	it with -DNO_DBUG.
 * 
 *  COPYRIGHT (c) 1995, Oliver Boehm (OBoehm@stgl.sel.alcatel.de)
 *	Permission to use, copy, modify, and to distribute this software and its
 *	documentation for any purpose is hereby granted without fee, provided
 *	that the above copyright notice appear in all copies and that both that
 *	copyright notice and this permission notice appear in supporting
 *	documentation. There is no representations about the suitability of
 *	this software for any purpose. This software is provided "as is" without
 *	express or implied warranty.
 * 
 *  HISTORY
 *	created:
 *	    16-May-95 by coreproducer O. Boehm
 *	modified:
 *	    Jul-95 by coreproducer O. Boehm
 *		- name "libnew.h" changed to "libonew.h"
 *	    Oct-95 by coreproducer O. Boehm
 *		- include <dbug.h> changed to <odebug.h> (__db_...functions not
 *		  faded out with -DNDEBUG)
 *	    Mar-96 by coreproducer O. Boehm
 *		- adaption to Solaris 2.5
 * 
 *  SEE ALSO
 *	libonew.c, lwp_new.c
 */



#ifndef LWP_NEW_H
#define LWP_NEW_H



#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <assert.h>

#include <ansi.h>



#if defined(unix) && !defined(NO_DBUG)	/* ------- use DBUG macros ---------- */

#include <odebug.h>				/* inculudes -> <dbug.h> */

#else					/* ------- fade out DBUG macros ----- */

#define DBUG_ENTER(a1)				/* -> see <dbug.h> */
#define DBUG_RETURN(a1) return(a1)
#define DBUG_VOID_RETURN return
#define DBUG_PROCESS(a1)

#endif					/* ------- end of DBUG section ------ */





#if defined (sun) && ! defined (NO_LWP)
    #define USE_LIBLWP			/* use the LWP(3) library */
#elif ! defined (NO_LWP)
    #define NO_LWP			/* don't use it */
#endif





#ifdef USE_LIBLWP		/* ---------	U S E   L W P    ------------ */



/*  LWP includes  */
#if defined(__svr4__) || defined (__SVR4)
    #include <pthread.h>
#else
    #include <lwp/lwp.h>
    #include <lwp/lwpmachdep.h>
    #include <lwp/stackdep.h>
#endif

/*  definements  */
#define MAXPRIO		2
#define	CHECK_HEAP	lwp_check_heap

/*  Prototype  */
void lwp_check_heap _((char* file, int line));



#else				/* ---------	N O   L W P    -------------- */



/*  definements  */
#define	CHECK_HEAP	ocheck_heap



#endif				/* ------------------------------------------ */





#endif	/* LWP_NEW_H */

