/*
 *  NAME
 *	lwp_new.c
 * 
 *  DESCRIPTION
 *	Starting some checks as own thread
 *
 *  BUGS
 *	No synchronisation forseen - the main thread may change the block list
 *	where the check tread is testing it.
 *
 *	On a MP machine there may be more than one check thread running at the
 *	same time.
 *
 *  NOTES
 *	This is my first experience on thread. It may be not the best code
 *	and not the best concept - but it's a first try. If you want to use
 *	it without the LWP library -> compile it with -DNO_LWP.
 *
 *	difficult to test
 * 
 *  VERSION $Revision: 3.2 $ ($Date: 1996/10/24 08:41:03 $)
 *
 *  COPYRIGHT (c) 1995, Oliver Boehm (OBoehm@stgl.sel.alcatel.de)
 *	Permission to use, copy, modify, and to distribute this software and its
 *	documentation for any purpose is hereby granted without fee, provided
 *	that the above copyright notice appear in all copies and that both that
 *	copyright notice and this permission notice appear in supporting
 *	documentation. There is no representations about the suitability of
 *	this software for any purpose. This software is provided "as is" without
 *	express or implied warranty.
 * 
 *  HISTORY
 *	created:
 *	    15-May-95 by coreproducer O. Boehm
 *	modified:
 *	    Jul-95 by coreproducer O. Boehm
 *		- include file "libnew.h" changed to "libonew.h"
 *	    Mar-96 by coreproducer O. Boehm
 *		- adaption to Solaris 2.5 (__svr4__ || __SVR4)
 *	    Oct-96 by coreproducer O. Boehm
 *		- adaption to RCS
 *
 *  SEE ALSO
 *	LWP(3) - Lightweight Processes (SunOS 4.1.3)
 *	THREAD(3T) - Thread Library (Solaris 2.5)
 */




#include "libonew.h"





#ifdef USE_LIBLWP			/* use LWP(3) */



#ifndef lint
static const char ident[]="$Id: lwp_new.c,v 3.2 1996/10/24 08:41:03 lve163d Exp $ Copyr O. Boehm";
#endif





#if !defined(__svr4__) && !defined (__SVR4)
stkalign_t  stack[1000+MINSTACKSZ];
#endif
char	    *caller = NULL;		/* source name which causes the check */





/* --------   P R O T O T Y P E S   ----------------------------------------- */

/*  local functions  */
#if defined(__svr4__) || defined (__SVR4)	/* ----- SVR4 (POSIX) ------- */
void task_check_heap _((int* line));
#else						/* ----- SunOS-4 (LWP) ------ */
void task_check_heap _((int line));
#endif	/* SunOS-4 (LWP) */





/*
 *  NAME
 *	lwp_check_heap
 *
 *  DESCRIPTION
 *	lwp_check_heap() calls ocheck_heap() on a Sun as an own thread using
 *	Sun's LWP library.
 */



#if defined(__svr4__) || defined (__SVR4)	/* ----- SVR4 (POSIX) ------- */

void lwp_check_heap HAS_2_PARAMS (char*, file, int, line)
{
    pthread_t tid;

    DBUG_ENTER ("lwp_check_heap");
    caller = file;
    if (pthread_create (&tid, NULL, (void *(*)(void *)) task_check_heap, &line)
    	    != 0) {
    	perror ("can't create a new thread");
    	task_check_heap (&line);
    } else {
    	pthread_join (tid, NULL);
    }
    DBUG_VOID_RETURN;
}

#else						/* ----- SunOS-4 (LWP) ------ */

void lwp_check_heap HAS_2_PARAMS (char*, file, int, line)
{
    thread_t tid;

    if (lwp_create (&tid, task_check_heap, MINPRIO, 0, STKTOP(stack), 1, line)
    	    != 0) {
    	lwp_perror ("can't create a new thread");
    	task_check_heap (line);
    } else {
    	lwp_resched (MINPRIO);
    }
    DBUG_VOID_RETURN;
}

#endif	/* SunOS-4 (LWP) */



#if defined(__svr4__) || defined (__SVR4)	/* ----- SVR4 (POSIX) ------- */

void task_check_heap HAS_1_PARAM (int*, line)
{
    DBUG_ENTER ("task_check_heap");
    ocheck_heap (caller, *line);
    DBUG_VOID_RETURN;
}

#else						/* ----- SunOS-4 (LWP) ------ */

void task_check_heap HAS_1_PARAM (int, line)
{
    DBUG_ENTER ("task_check_heap");
    ocheck_heap (caller, line);
    DBUG_VOID_RETURN;
}

#endif	/* SunOS-4 (LWP) */





#endif	/* USE_LIBLWP */
