/*
 *  NAME
 *	onew.h ($Revision: 2.4 $)
 * 
 *  DESCRIPTION
 *	This file must be included if the functions in "onew.c" should be
 *	taken. It changes via #define the names for malloc, calloc, realloc and
 *	free into the name used in new.c (new, cnew, renew and delete).
 *	The replacement is made only if the NDEBUG flag is not defined.
 * 
 *  COPYRIGHT (c) 1991-2000 O. Boehm (boehm@ba-stuttgart.de)
 *	Permission to use, copy, modify, and to distribute this software and its
 *	documentation for any purpose is hereby granted without fee, provided
 *	that the above copyright notice appear in all copies and that both that
 *	copyright notice and this permission notice appear in supporting
 *	documentation. There is no representations about the suitability of
 *	this software for any purpose. This software is provided "as is" without
 *	express or implied warranty.
 * 
 *  HISTORY
 *	created:
 *	    (c) 20-Nov-91 O. Boehm
 *	modified:
 *	    12-Jan-93 by coreproducer O. Boehm
 *		- file name and line number added to error message
 *		- warning "MALLOC redefined" avoided when used together with
 *		  "util.h"
 *	    19-Apr-93 by coreproducer O. Boehm
 *		- malloc(0) now produces a warning!
 *		- ansi.h now included
 *	    26-Oct-93 by coreproducer O. Boehm
 *		- protoypes for mnew(), ... corrected
 *		- token after '#endif' now a comment
 *	    18-Jul-94 by coreproducer O. Boehm
 *		- <malloc.h> included now
 *		- mdelete(d) changed to odelete(d) because of conflicts with a
 *		  ftp library
 *		- support of strdup()
 *	    May-95 by coreproducer O. Boehm
 *		- new functions check_heap(), check_pointer() added
 *	    Jul-95 by coreproducer O. Boehm
 *		- name changed to "onew.h" to avoid conflicts with Borland
 *		  include files
 *		- prototypes for allog_begin(), allog_end() added
 *	    Feb-95 by coreproducer O. Boehm
 *		- NONEW #defined if the original "malloc()" is wanted
 *	    Feb-2000 by O. Boehm
 *		- strdup is a macro under Linux
 * 
 *  SEE ALSO
 *	new.c
 *	/usr/adm_sel/lib/include/util.h
 *	/usr/adm_sel/lib/include/ansi.h
 */



#ifndef NEW_H
#define NEW_H



#include <malloc.h>		/* must be included before */
#include <string.h>		/*	- " -		   */

#include <ansi.h>



#if !defined(NDEBUG) && !defined(NONEW)


#define malloc(size)		mnew(size,__FILE__,__LINE__)
#define calloc(nobj,size)	cnew(nobj,size,__FILE__,__LINE__)
#define realloc(p,size)		renew(p,size,__FILE__,__LINE__)
#define free(p)			odelete(p,__FILE__,__LINE__)
#define freed()			odeleted(__FILE__,__LINE__)
#define check_pointer(p)	ocheck_pointer(p,__FILE__,__LINE__,NULL)
#define check_heap()		ocheck_heap(__FILE__,__LINE__)
#define allog_begin(logname)	oallog_begin(logname,__FILE__,__LINE__)
#define allog_end()		oallog_end(__FILE__,__LINE__)

void *mnew _((size_t size, char* file, int line));
void *cnew _((size_t nobj, size_t size, char* file, int line));
void *renew _((void *p, size_t size, char* file, int line));
void odelete _((void *p, char* file, int line));
void odeleted _((char* file, int line));
int  ocheck_pointer _((void *p, char* file, int line, char* msg));
int  ocheck_heap _((char* file, int line));
void oallog_begin _((const char *logname, char* file, int line));
void oallog_end _((char* file, int line));



/*  -------   S U P P O R T E D   S T R I N G   F U N C T I O N S   --------  */
/*  ----------------------   (see also <string.h>)   -----------------------  */
/*
 *  NOTE
 *	To avoid conflicts with the standard library about the range of
 *	resolving of externals the functions are preceeded by an 'o' and
 *	#define'd to that name.
 */

#ifdef	strdup
#undef	strdup
#endif
#define strdup(s)		ostrdup(s,__FILE__,__LINE__)
char *ostrdup _((char *s, char* file, int line));
				/* I would prefer "const char" as parameter   */
				/* but it's defined in this way in <string.h> */



#else	/* NDEBUG || NONEW */



#define freed()
#define check_pointer(p)
#define check_heap()
#define allog_begin(x)



#endif	/* NDEBUG || NONEW */





#ifndef	MALLOC

#define MALLOC(type)		(type*)malloc(sizeof(type))
#define CALLOC(type,nobj)	(type*)calloc(nobj,sizeof(type))
#define REALLOC(type,p,nobj)	(type*)realloc(p,sizeof(type)*(nobj))
#define FREE(p)			if(p){free(p);p=NULL;}

#ifdef	__cplusplus
#define DELETE(ptr)		if (ptr) { delete(ptr); }
#else
#define NEW(type)		(type*)malloc(sizeof (type))
#define DELETE(ptr)		if (ptr) { free(ptr); ptr = NULL; }
#endif

#endif	/* MALLOC */






#include <string.h>		/* must be included AFTER this definitions */
				/* to avoid conflicts with strdup()	   */



#endif	/* NEW_H */

