/*
 *  NAME
 *	tnew.c
 * 
 *  DESCRIPTION
 *	Testprogramm for the libonew library
 *
 *  OPTIONS
 *	-l <logfile>	log messages into <logfile>
 * 
 *  BUGS
 *	The restoration of a corrupted heap is not tested.
 * 
 *  VERSION $Revision: 2.2 $ ($Date: 1995/08/08 08:20:30 $)
 *
 *  COPYRIGHT (c) 1995, Oliver Boehm (OBoehm@stgl.sel.alcatel.de)
 *	Permission to use, copy, modify, and to distribute this software and its
 *	documentation for any purpose is hereby granted without fee, provided
 *	that the above copyright notice appear in all copies and that both that
 *	copyright notice and this permission notice appear in supporting
 *	documentation. There is no representations about the suitability of
 *	this software for any purpose. This software is provided "as is" without
 *	express or implied warranty.
 * 
 *  HISTORY
 *	(c)reated:
 *	    15-May-95 by coreproducer O. Boehm
 *	modified:
 *	    Jul-95 by coreproducer O. Boehm
 *		- include file "new.h" changed to "onew.h"
 *		- option '-l' added
 *
 *  SEE ALSO
 *	libonew.c
 */



#ifndef lint
static const char ident[]="$Id: tnew.c,v 2.2 1995/08/08 08:20:30 lve163d Exp $ Copyr O. Boehm";
#endif



#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <ansi.h>
#include <odebug.h>

#include "onew.h"



main HAS_2_PARAMS (int, argc, char **, argv)
{
    char *p2, *p3, *p4, *p5, *p6, *p7;
    struct s_list {
    	struct s_list	*next;
    	char		data[10000];
    } l0, *pl, *pl2;
    unsigned i;
    
    DBUG_ENTER ("main");
    DBUG_PROCESS (argv[0]);

    /* parse arguments	 */
    for (i = 1; i < argc; i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
#ifndef NDEBUG
		case '#':   DBUG_PUSH (&(argv[i][2]));
			    break;
#else
		case '#':   (void) fprintf (
				stderr,
				"Sorry, option \"%s\" implemented only in the"
				"test version!\n",
				argv[i]
			    );
			    break;
#endif
		case 'l':   allog_begin (argv[++i]);
			    break;
		default:    (void) fprintf (
				stderr,
				"\007unknown option \"%s\"!\n",
				argv[i]
			    );
			    DBUG_RETURN (EXIT_FAILURE);
			    /*
			    //  to avoid the warning: "statement after return
			    //  not reached" the following 'break' is #ifdef'd
			    //  out for sun
			    */
			    #ifndef sun
				/* NOTREACHED */
				break;
			    #endif
	    }
	}
    }

    /*	require too small memory  */
    p2 = malloc (0);
    p2 = CALLOC (char, 5);
    strcpy (p2, "hello");		/* "hello" requires 6 bytes! */

    /*  reallocate memory and forget pointer p3 to switch to new location  */
    p3 = p2 + 1;
    printf ("p2 = \"%s\", p3 = \"%s\"\n", p2, p3);
    p2 = REALLOC (char, p2, 6);
    printf ("p2 = \"%s\", p3 = \"%s\"\n", p2, p3);
    free (p2);
    
    /*	require memory, but don't give back  */
    p3 = malloc(1);
    
    /*	invalid pointers to free/reallocate  */
    p4 = p3 + 1;
    free (p4);
    free (p4+1);
    free (p4+3);
    p4 = REALLOC (char, p4, 2);
    
    /*	assumption that the block is cleared (with zeros)  */
    p5 = (char *) malloc (2);
    printf ("p5 = \"%s\"\n", p5);
    
    /*	access to a freed block	 */
    p6 = p2;
    printf ("p6 = \"%s\"\n", p6);
    free (p6);
    
    /*  strdup test  */
    p7 = strdup ("hello");
    printf ("p7 = strdup (\"hello\") = \"%s\"\n", p7);
    free (p7);
    
    /*  heap test  */
    (void) printf ("heap test");
    fflush (stdout);
    l0.next = NULL;
    pl = &l0;
    for (i = 0, pl = &l0; i < 50; pl = pl->next, i++) {
    	pl->next = malloc (sizeof (struct s_list));
    	check_heap();
    	printf (".");
    	fflush (stdout);
    	if (pl->next == NULL) {
    	    break;
    	}
    }
    printf ("\n");
    
    /*  corrupt the heap  */
    memset (((char*)l0.next - 24), 0x12, 24);
    check_heap();
    
    /*  free the heap  */
    pl = pl->next = NULL;
    for (pl = &l0; pl != NULL; pl = pl2) {
        pl2 = pl->next;
        free (pl);
    }
    
    /*	end  */
    freed ();
    DBUG_RETURN (EXIT_SUCCESS);
}
